// mpicc Ex2-SendRecv.c -Wall -o Ex2-SendRecv.x

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define MIN(x, y) (x) < (y) ? (x) : (y)
#define N 20

int main( int argc, char *argv[] )
{   
    int myrank, nranks;
    int randomnums[N];
    int local_max, global_max, *global_maxs;
    int root = 0;
    int tag = 0;
    int i;

    MPI_Init(&argc,&argv);

    MPI_Comm_rank( MPI_COMM_WORLD, &myrank );
    MPI_Comm_size( MPI_COMM_WORLD, &nranks );

    // Generate N random integers in each process
    srand( myrank );
    for ( i = 0; i < N; i++ )
        randomnums[ i ] = (lrand48() % 199) - 99;

    printf("[%d]: [", myrank);
    for ( i = 0; i < N; i++ )
        printf(" %d,", randomnums[i]);
    printf("]\n");

    // Get local maximum
    local_max = randomnums[0];
    for ( i = 1; i < N; i++ )
        if (randomnums[i] > local_max)
            local_max = randomnums[i];
    printf("[%d]: Local max: %d\n", myrank, local_max);
    
    // Gather the local maximum from each process in root
    if ( myrank == root )
    {
        global_maxs = (int *)malloc( nranks*sizeof(int) );
        global_maxs[ myrank ] = local_max;
        for ( i = 0; i < nranks; i++ )
        {
            if ( i == myrank )
                continue;
            MPI_Recv( &global_maxs[i], 1, MPI_INT, i, tag, 
                      MPI_COMM_WORLD, MPI_STATUS_IGNORE );
        }

        // Find the global maximum
        global_max = global_maxs[0];
        for ( i = 1; i < nranks; i++ )
            if (global_maxs[i] > global_max)
                global_max = global_maxs[i];

        printf("Maximum: %d\n", global_max);
        printf("Containers:\n");
        for ( i = 0; i < N; i++ )
        {
            if ( global_maxs[i] == global_max )
                printf("\t Rank %d\n", i );
        }
        free( global_maxs );
    }
    else
    {
        MPI_Send( &local_max, 1, MPI_INT, root, tag, 
                  MPI_COMM_WORLD );
    }


    MPI_Finalize();

    return 0;
}
