// mpicc Ex2-gather.c -Wall -o Ex2-gather.x

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define N 5

int main( int argc, char *argv[] )
{
    int rank, nprocs;
    int randomnums[N];
    int local_max, *global_maxs;
    int root = 0;
    int i;

    MPI_Init(&argc,&argv);

    MPI_Comm_rank( MPI_COMM_WORLD, &rank );
    MPI_Comm_size( MPI_COMM_WORLD, &nprocs );

    // Generate N random integers
    srand( rank );
    for ( i = 0; i < N; i++ )
        randomnums[ i ] = (lrand48() % 199) - 99;

    printf("[%d]: [", rank);
    for ( i = 0; i < N; i++ )
        printf(" %d,", randomnums[i]);
    printf("]\n");

    // Compute the local maximum
    local_max = randomnums[0];
    for ( i = 1; i < N; i++ )
        if (randomnums[i] > local_max)
            local_max = randomnums[i];
    printf("[%d]: Local max: %d\n", rank, local_max);
    
    // Gather the local maximum from each process in root
    global_maxs = (int *)malloc( nprocs*sizeof(int) );
    MPI_Gather( &local_max, 1, MPI_INT, global_maxs, 1, MPI_INT,
                root, MPI_COMM_WORLD);

    if ( rank  == 0 )
    {
        // Find the global maximum
        int global_max = global_maxs[0];
        for ( i = 1; i < nprocs; i++ )
            if (global_maxs[i] > global_max)
                global_max = global_maxs[i];

        // Print the global maximum and the owner(s)
        printf("Maximum: %d\n", global_max );
        for ( i = 0; i < nprocs; i++ )
            if ( global_maxs[i]  == global_max )
                printf("[%d] contains the maximum\n", i );
    }

    free( global_maxs );

    MPI_Finalize();
    return 0;
}


