// mpicc Ex4.c -Wall -o Ex4.x

#include <stdio.h>
#include <stdlib.h>
#include "mpi.h"

#define MIN(x, y) (x) < (y) ? (x) : (y)

int myMPI_Reduce( const void *sendbuf, void *recvbuf, int count,
                  int root, MPI_Comm comm )
{
    int myrank, nprocs;
    int tag = 0;

    MPI_Comm_rank( comm, &myrank );
    MPI_Comm_size( comm, &nprocs );

    if ( myrank == root )
    {
        int i, rank;
        double *tmpbuf = (double *) malloc (count * sizeof( double ));

        for ( i = 0; i < count; i++ )
            tmpbuf[i] = ((double *)sendbuf)[i];

        for ( rank = 0; rank < nprocs; rank++ )
        {
            if ( rank == myrank )
                continue;
            MPI_Recv( recvbuf, count, MPI_DOUBLE, rank, tag, comm, MPI_STATUS_IGNORE );
            for ( i = 0; i < count; i++ )
                tmpbuf[i] = MIN( ((double *)tmpbuf)[i], ((double *)recvbuf)[i] );
        }

        for ( i = 0; i < count; i++ )
            ((double *)recvbuf)[i] = tmpbuf[i];

        free( tmpbuf );
    }
    else
    {
        MPI_Send( sendbuf, count, MPI_DOUBLE, root, tag, comm );
    }

    return 0;
}

int main( int argc, char *argv[] )
{   
    int i;
    int ROOT = 0;
    int myrank, nprocs;
    double sendbuf[1], recvbuf[1];

    MPI_Init(&argc,&argv);

    MPI_Comm_rank( MPI_COMM_WORLD, &myrank );
    MPI_Comm_size( MPI_COMM_WORLD, &nprocs );
    
    srand( myrank*nprocs );
    sendbuf[ 0 ] = ((double)rand()) / RAND_MAX;
        
    for ( i = 0; i < 1; i++ )
        printf( "[%d] %d: %f\n", myrank, i, sendbuf[i] );
    
    myMPI_Reduce( (const void *)sendbuf, (void *)recvbuf, 1, ROOT, MPI_COMM_WORLD );

    if ( myrank == ROOT )
    {
        printf( "MIN:\n" );
        for ( i = 0; i < 1; i++ )
            printf( "[%d] %d: %f\n", myrank, i, recvbuf[i] );
    }

    MPI_Finalize();
    return 0;
}


